/*
 * @(#)ClasspathResourceLoader.java				Jun 1, 2012 11:45:00 PM
 *
 * Copyright 2012 Radeon Systems
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.radeonsys.data.querystore.support.loader.classpath;

import java.io.InputStream;
import java.io.Reader;
import java.net.URL;
import java.nio.charset.Charset;

import com.google.common.base.Optional;
import com.google.common.base.Preconditions;
import com.google.common.base.Strings;
import com.google.common.io.InputSupplier;
import com.google.common.io.Resources;
import com.radeonsys.data.querystore.support.loader.AbstractResourceLoader;
import com.radeonsys.data.querystore.support.loader.ResourceLoader;
import com.radeonsys.data.querystore.support.loader.ResourceNotFoundException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Implements the {@link ResourceLoader} interface to provide loading resources off the
 * the application classpath.
 * 
 * <p>This implementation supports loading both text resources as well as binary. Therefore
 * provides implementations of the {@link ResourceLoader#getReadableResource(String)} method,
 * {@link ResourceLoader#getReadableResource(String, Charset)} method and the 
 * {@link ResourceLoader#getBinaryResource(String)} method.</p>
 * 
 * @see ResourceLoader
 * @see Charset
 * @see Reader
 * @see InputStream
 * @see InputSupplier
 * 
 * @author oddjobsman at radeonsys dot com
 * @since 1.0
 */
public class ClasspathResourceLoader extends AbstractResourceLoader {

    private final Logger logger = LoggerFactory.getLogger(ClasspathResourceLoader.class);

	/** 
	 * Returns a factory that provides a {@link Reader} implementation over the resource at the
     * specified location on the classpath using the specified character set to decode
     * the contents of the resource
     *
     * @param resourceLocation	the location of the resource on the classpath
     * @param charset           reference to an instance of {@link Charset} that represents the
     *                          character set used to decode the contents of the resource
     *
     * @return  reference to an instance of {@link InputSupplier} which provides instances of
     *          {@link Reader} over the resource at the specified location on the classpath
	 */
	@Override
	public InputSupplier<? extends Reader> getReadableResource(String resourceLocation, Charset charset) {
		return Resources.newReaderSupplier(getResourceUrl(resourceLocation), charset);
	}
	
	/** 
	 * Returns a factory that provides a {@link InputStream} implementation over the resource at the
     * specified location on the classpath
     *
     * @param resourceLocation	the location of the resource on the classpath
     *
     * @return  reference to an instance of {@link InputSupplier} which provides instances of
     *          {@link InputStream} over the resource at the specified location on the classpath
	 */
	@Override
	public InputSupplier<? extends InputStream> getBinaryResource(String resourceLocation) {
		return Resources.newInputStreamSupplier(getResourceUrl(resourceLocation));
	}

	/**
	 * Returns the location of the resource resolved over the classpath
	 * 
	 * @param resourceLocation	the location of the resource on the classpath
	 * 
	 * @return	reference to an instance of {@link URL} which represents the location of the
	 * 			resource resolved over the classpath
	 */
	private URL getResourceUrl(String resourceLocation) {
		String sanitizedResourceLocation = sanitizeResourceLocation(resourceLocation);
		
		// attempt to find the resource using context class loader
		Optional<URL> optionalResourceUrl = getResourceUrl(sanitizedResourceLocation, 
				Thread.currentThread().getContextClassLoader());

        // if not found, attempt to find the resource using ClasspathResourceLoader's class loader
        if (!optionalResourceUrl.isPresent())
            optionalResourceUrl = getResourceUrl(sanitizedResourceLocation, ClasspathResourceLoader.class.getClassLoader());

		if (!optionalResourceUrl.isPresent()) {
            if (logger.isWarnEnabled())
                logger.warn(String.format("Unable to find classpath resource: %s", resourceLocation));

            // if still not found, we cannot locate the resource
            throw new ResourceNotFoundException(resourceLocation, buildResourceNotFoundMessage(resourceLocation));
        }

        if (logger.isDebugEnabled())
            logger.debug(String.format("Retrieved classpath resource: %s", resourceLocation));

        return optionalResourceUrl.get();
	}
	
	/**
	 * Returns a sanitized version of the specified resource location
	 * 
	 * @param resourceLocation	the resource location to be sanitized
	 * 
	 * @return	a sanitized version of the specified resource location
	 */
	private String sanitizeResourceLocation(String resourceLocation) {
		Preconditions.checkArgument(!Strings.isNullOrEmpty(resourceLocation), 
				"A valid resource location must be specified");
		
		if (resourceLocation.charAt(0) == '/')
			return resourceLocation.substring(1);
		return resourceLocation;
	}

	/**
	 * Returns the location of the resource resolved over the classpath using the specified
	 * class loader
	 * 
	 * @param resourceLocation	the location of the resource on the classpath
	 * @param classLoader		reference to an instance of {@link ClassLoader} to use to resolve 
	 * 							the location of the resource
	 * 
	 * @return	reference to an instance of {@link Optional} which provides a {@link URL}
	 * 			that represent the location of the resource resolved over the classpath using
	 * 			the specified {@link ClassLoader} 
	 */
	private Optional<URL> getResourceUrl(String resourceLocation, ClassLoader classLoader) {
		return Optional.fromNullable(classLoader.getResource(resourceLocation));
	}

	/**
	 * Returns an error message for a resource not being found on the classpath at the specified 
	 * location
	 * 
	 * @param resourceLocation	the location of the resource on the classpath
	 * 
	 * @return	an error message for a resource not being found on the classpath at the specified 
	 * 			location
	 */
	private String buildResourceNotFoundMessage(String resourceLocation) {
		return String.format("Unable to find resource on classpath at the specified location: %s", 
				resourceLocation);
	}
}
